@@ -25,13 +25,19 @@
 ** Commercial non-GPL licensing of this software is possible.
 ** For more info contact Nero AG through Mpeg4AAClicense@nero.com.
 **
-** $Id: main.c,v 1.85 2008/09/22 17:55:09 menno Exp $
+** $Id: main.c,v 1.89 2015/01/19 09:46:12 knik Exp $
 **/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
+#ifndef __MINGW32__
 #define off_t __int64
+#endif
 #else
 #include <time.h>
 #endif
@@ -44,9 +50,16 @@
 #include <getopt.h>
 
 #include <neaacdec.h>
-#include <mp4ff.h>
 
 #include "audio.h"
+#include "mp4read.h"
+
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
+#else
+# include "getopt.h"
+# include "getopt.c"
+#endif
 
 #ifndef min
 #define min(a,b) ( (a) < (b) ? (a) : (b) )
@@ -130,11 +143,43 @@ static int fill_buffer(aac_buffer *b)
 
 static void advance_buffer(aac_buffer *b, int bytes)
 {
-    b->file_offset += bytes;
-    b->bytes_consumed = bytes;
-    b->bytes_into_buffer -= bytes;
-	if (b->bytes_into_buffer < 0)
-		b->bytes_into_buffer = 0;
+    while ((b->bytes_into_buffer > 0) && (bytes > 0))
+    {
+	int chunk = min(bytes, b->bytes_into_buffer);
+	
+	bytes -= chunk;
+	b->file_offset += chunk;
+	b->bytes_consumed = chunk;
+	b->bytes_into_buffer -= chunk;
+
+	if (b->bytes_into_buffer == 0)
+	    fill_buffer(b);
+    }
+}
+
+static void lookforheader(aac_buffer *b)
+{
+  int i = 0;
+	while (!b->at_eof )
+	{
+       		if (b->bytes_into_buffer > 4)
+        	{
+		    if( ((b->buffer[0+i] == 0xff) && ((b->buffer[1+i] & 0xf6) == 0xf0)) ||
+       			 (b->buffer[0+i] == 'A'    && b->buffer[1+i] == 'D' && b->buffer[2+i] == 'I' && b->buffer[3+i] == 'F'))
+			{
+				fill_buffer(b);
+				break;
+			} else {
+				i++;
+    				b->file_offset       += 1;
+    				b->bytes_consumed    += 1;
+    				b->bytes_into_buffer -= 1;
+			}
+		} else {
+			fill_buffer(b);
+			i = 0;
+		}
+	}
 }
 
 static int adts_sample_rates[] = {96000,88200,64000,48000,44100,32000,24000,22050,16000,12000,11025,8000,7350,0,0,0};
@@ -387,6 +432,7 @@ static void usage(void)
     faad_fprintf(stdout, " -w    Write output to stdio instead of a file.\n");
     faad_fprintf(stdout, " -g    Disable gapless decoding.\n");
     faad_fprintf(stdout, " -q    Quiet - suppresses status messages.\n");
+    faad_fprintf(stdout, " -j X  Jump - start output X seconds into track (MP4 files only).\n");
     faad_fprintf(stdout, "Example:\n");
     faad_fprintf(stdout, "       %s infile.aac\n", progName);
     faad_fprintf(stdout, "       %s infile.mp4\n", progName);
@@ -424,6 +470,8 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
     float length = 0;
 
     int first_time = 1;
+    int retval;
+    int streaminput = 0;
 
     aac_buffer b;
 
@@ -439,17 +487,39 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
         }
     }
 
-    b.infile = fopen(aacfile, "rb");
-    if (b.infile == NULL)
+    if (0 == strcmp(aacfile, "-"))
     {
-        /* unable to open file */
-        faad_fprintf(stderr, "Error opening file: %s\n", aacfile);
-        return 1;
+	b.infile = stdin;
+#ifdef _WIN32
+        setmode(fileno(stdin), O_BINARY);
+#endif
+
+    } else
+    {
+    	b.infile = fopen(aacfile, "rb");
+    	if (b.infile == NULL)
+    	{
+    	    /* unable to open file */
+    	    faad_fprintf(stderr, "Error opening file: %s\n", aacfile);
+    	    return 1;
+    	}
     }
 
-    fseek(b.infile, 0, SEEK_END);
-    fileread = ftell(b.infile);
-    fseek(b.infile, 0, SEEK_SET);
+    retval = fseek(b.infile, 0, SEEK_END);
+#ifdef _WIN32
+	if (0 == strcmp(aacfile, "-")) {
+	  retval = -1;
+	}
+#endif
+    if (retval )
+    {
+	 faad_fprintf(stderr, "Input not seekable %s\n", aacfile);
+	 fileread = -1;
+         streaminput = 1;
+    } else {
+    	fileread = ftell(b.infile);
+    	fseek(b.infile, 0, SEEK_SET);
+    };
 
     if (!(b.buffer = (unsigned char*)malloc(FAAD_MIN_STREAMSIZE*MAX_CHANNELS)))
     {
@@ -494,19 +564,39 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
 
     /* get AAC infos for printing */
     header_type = 0;
+    if (streaminput == 1 )
+	lookforheader(&b);
+
     if ((b.buffer[0] == 0xFF) && ((b.buffer[1] & 0xF6) == 0xF0))
     {
-        adts_parse(&b, &bitrate, &length);
-        fseek(b.infile, tagsize, SEEK_SET);
+        if (streaminput ==1)
+	{
+	    int frames, frame_length;
+	    int samplerate;
+	    float frames_per_sec, bytes_per_frame;
+	    samplerate = adts_sample_rates[(b.buffer[2]&0x3c)>>2];
+	    frame_length = ((((unsigned int)b.buffer[3] & 0x3)) << 11)
+	        | (((unsigned int)b.buffer[4]) << 3) | (b.buffer[5] >> 5);
 
-        bread = fread(b.buffer, 1, FAAD_MIN_STREAMSIZE*MAX_CHANNELS, b.infile);
-        if (bread != FAAD_MIN_STREAMSIZE*MAX_CHANNELS)
-            b.at_eof = 1;
-        else
-            b.at_eof = 0;
-        b.bytes_into_buffer = bread;
-        b.bytes_consumed = 0;
-        b.file_offset = tagsize;
+	    frames_per_sec = (float)samplerate/1024.0f;
+	    bytes_per_frame = (float)frame_length/(float)(1000);
+	    bitrate = (int)(8. * bytes_per_frame * frames_per_sec + 0.5);
+	    length = 1;
+	    faad_fprintf(stderr, "Streamed input format  samplerate %d channels %d.\n",samplerate,channels);
+
+	} else {
+		adts_parse(&b, &bitrate, &length);
+        	fseek(b.infile, tagsize, SEEK_SET);
+
+        	bread = fread(b.buffer, 1, FAAD_MIN_STREAMSIZE*MAX_CHANNELS, b.infile);
+        	if (bread != FAAD_MIN_STREAMSIZE*MAX_CHANNELS)
+        	    b.at_eof = 1;
+        	else
+        	    b.at_eof = 0;
+        	b.bytes_into_buffer = bread;
+        	b.bytes_consumed = 0;
+        	b.file_offset = tagsize;
+	}
 
         header_type = 1;
     } else if (memcmp(b.buffer, "ADIF", 4) == 0) {
@@ -538,7 +628,8 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
         if (b.buffer)
             free(b.buffer);
         NeAACDecClose(hDecoder);
-        fclose(b.infile);
+        if (b.infile != stdin)
+            fclose(b.infile);
         return 1;
     }
     advance_buffer(&b, bread);
@@ -564,7 +655,8 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
     if (infoOnly)
     {
         NeAACDecClose(hDecoder);
-        fclose(b.infile);
+        if (b.infile != stdin)
+            fclose(b.infile);
         if (b.buffer)
             free(b.buffer);
         return 0;
@@ -621,7 +713,8 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
                     if (b.buffer)
                         free(b.buffer);
                     NeAACDecClose(hDecoder);
-                    fclose(b.infile);
+                    if (b.infile != stdin)
+                        fclose(b.infile);
                     return 0;
                 }
             } else {
@@ -662,7 +755,8 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
         fclose(adtsFile);
     }
 
-    fclose(b.infile);
+    if (b.infile != stdin)
+        fclose(b.infile);
 
     if (!first_time && !adts_out)
         close_audio_file(aufile);
@@ -673,35 +767,6 @@ static int decodeAACfile(char *aacfile, char *sndfile, char *adts_fn, int to_std
     return frameInfo.error;
 }
 
-static int GetAACTrack(mp4ff_t *infile)
-{
-    /* find AAC track */
-    int i, rc;
-    int numTracks = mp4ff_total_tracks(infile);
-
-    for (i = 0; i < numTracks; i++)
-    {
-        unsigned char *buff = NULL;
-        int buff_size = 0;
-        mp4AudioSpecificConfig mp4ASC;
-
-        mp4ff_get_decoder_config(infile, i, &buff, &buff_size);
-
-        if (buff)
-        {
-            rc = NeAACDecAudioSpecificConfig(buff, buff_size, &mp4ASC);
-            free(buff);
-
-            if (rc < 0)
-                continue;
-            return i;
-        }
-    }
-
-    /* can't decode this */
-    return -1;
-}
-
 static const unsigned long srates[] =
 {
     96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000,
@@ -710,19 +775,17 @@ static const unsigned long srates[] =
 
 static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_stdout,
                   int outputFormat, int fileType, int downMatrix, int noGapless,
-                  int infoOnly, int adts_out, float *song_length)
+                  int infoOnly, int adts_out, float *song_length, float seek_to)
 {
     int track;
     unsigned long samplerate;
     unsigned char channels;
     void *sample_buffer;
 
-    mp4ff_t *infile;
-    long sampleId, numSamples;
+    long sampleId, startSampleId;
 
     audio_file *aufile;
 
-    FILE *mp4File;
     FILE *adtsFile;
     unsigned char *adtsData;
     int adtsDataSize;
@@ -732,9 +795,6 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
     NeAACDecFrameInfo frameInfo;
     mp4AudioSpecificConfig mp4ASC;
 
-    unsigned char *buffer;
-    int buffer_size;
-
     char percents[200];
     int percent, old_percent = -1;
 
@@ -742,19 +802,25 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
 
     /* for gapless decoding */
     unsigned int useAacLength = 1;
-    unsigned int initial = 1;
     unsigned int framesize;
-    unsigned long timescale;
+    unsigned decoded;
 
+    if (strcmp(mp4file, "-") == 0 ) {
+        faad_fprintf(stderr, "Cannot open stdin for MP4 input \n");
+        return 1;
+    }
 
-    /* initialise the callback structure */
-    mp4ff_callback_t *mp4cb = malloc(sizeof(mp4ff_callback_t));
-
-    mp4File = fopen(mp4file, "rb");
-    mp4cb->read = read_callback;
-    mp4cb->seek = seek_callback;
-    mp4cb->user_data = mp4File;
-
+    if (!quiet)
+    {
+        mp4config.verbose.header = 1;
+        mp4config.verbose.tags = 1;
+    }
+    if (mp4read_open(mp4file))
+    {
+        /* unable to open file */
+        faad_fprintf(stderr, "Error opening file: %s\n", mp4file);
+        return 1;
+    }
 
     hDecoder = NeAACDecOpen();
 
@@ -775,52 +841,27 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
         }
     }
 
-    infile = mp4ff_open_read(mp4cb);
-    if (!infile)
-    {
-        /* unable to open file */
-        faad_fprintf(stderr, "Error opening file: %s\n", mp4file);
-        return 1;
-    }
-
-    if ((track = GetAACTrack(infile)) < 0)
-    {
-        faad_fprintf(stderr, "Unable to find correct AAC sound track in the MP4 file.\n");
-        NeAACDecClose(hDecoder);
-        mp4ff_close(infile);
-        free(mp4cb);
-        fclose(mp4File);
-        return 1;
-    }
-
-    buffer = NULL;
-    buffer_size = 0;
-    mp4ff_get_decoder_config(infile, track, &buffer, &buffer_size);
-
-    if(NeAACDecInit2(hDecoder, buffer, buffer_size,
+    if(NeAACDecInit2(hDecoder, mp4config.asc.buf, mp4config.asc.size,
                     &samplerate, &channels) < 0)
     {
         /* If some error initializing occured, skip the file */
         faad_fprintf(stderr, "Error initializing decoder library.\n");
         NeAACDecClose(hDecoder);
-        mp4ff_close(infile);
-        free(mp4cb);
-        fclose(mp4File);
+        mp4read_close();
         return 1;
     }
 
-    timescale = mp4ff_time_scale(infile, track);
     framesize = 1024;
     useAacLength = 0;
+    decoded = 0;
 
-    if (buffer)
+    if (mp4config.asc.size)
     {
-        if (NeAACDecAudioSpecificConfig(buffer, buffer_size, &mp4ASC) >= 0)
+        if (NeAACDecAudioSpecificConfig(mp4config.asc.buf, mp4config.asc.size, &mp4ASC) >= 0)
         {
             if (mp4ASC.frameLengthFlag == 1) framesize = 960;
             if (mp4ASC.sbr_present_flag == 1) framesize *= 2;
         }
-        free(buffer);
     }
 
     /* print some mp4 file info */
@@ -829,74 +870,38 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
         char *tag = NULL, *item = NULL;
         int k, j;
         char *ot[6] = { "NULL", "MAIN AAC", "LC AAC", "SSR AAC", "LTP AAC", "HE AAC" };
-        long samples = mp4ff_num_samples(infile, track);
-        float f = 1024.0;
         float seconds;
-        if (mp4ASC.sbr_present_flag == 1)
-        {
-            f = f * 2.0;
-        }
-        seconds = (float)samples*(float)(f-1.0)/(float)mp4ASC.samplingFrequency;
+        seconds = (float)mp4config.samples/(float)mp4ASC.samplingFrequency;
 
         *song_length = seconds;
 
         faad_fprintf(stderr, "%s\t%.3f secs, %d ch, %d Hz\n\n", ot[(mp4ASC.objectTypeIndex > 5)?0:mp4ASC.objectTypeIndex],
             seconds, mp4ASC.channelsConfiguration, mp4ASC.samplingFrequency);
-
-#define PRINT_MP4_METADATA
-#ifdef PRINT_MP4_METADATA
-        j = mp4ff_meta_get_num_items(infile);
-        for (k = 0; k < j; k++)
-        {
-            if (mp4ff_meta_get_by_index(infile, k, &item, &tag))
-            {
-                if (item != NULL && tag != NULL)
-                {
-                    faad_fprintf(stderr, "%s: %s\n", item, tag);
-                    free(item); item = NULL;
-                    free(tag); tag = NULL;
-                }
-            }
-        }
-        if (j > 0) faad_fprintf(stderr, "\n");
-#endif
     }
 
     if (infoOnly)
     {
         NeAACDecClose(hDecoder);
-        mp4ff_close(infile);
-        free(mp4cb);
-        fclose(mp4File);
+        mp4read_close();
         return 0;
     }
 
-    numSamples = mp4ff_num_samples(infile, track);
+    startSampleId = 0;
+    if (seek_to > 0.1)
+        startSampleId = (int64_t)(seek_to * mp4config.samplerate / framesize);
 
-    for (sampleId = 0; sampleId < numSamples; sampleId++)
+    mp4read_seek(startSampleId);
+    for (sampleId = startSampleId; sampleId < mp4config.frame.ents; sampleId++)
     {
         int rc;
         long dur;
         unsigned int sample_count;
         unsigned int delay = 0;
 
-        /* get acces unit from MP4 file */
-        buffer = NULL;
-        buffer_size = 0;
+        if (mp4read_frame())
+            break;
 
-        dur = mp4ff_get_sample_duration(infile, track, sampleId);
-        rc = mp4ff_read_sample(infile, track, sampleId, &buffer,  &buffer_size);
-        if (rc == 0)
-        {
-            faad_fprintf(stderr, "Reading from MP4 file failed.\n");
-            NeAACDecClose(hDecoder);
-            mp4ff_close(infile);
-            free(mp4cb);
-            fclose(mp4File);
-            return 1;
-        }
-
-        sample_buffer = NeAACDecDecode(hDecoder, &frameInfo, buffer, buffer_size);
+        sample_buffer = NeAACDecDecode(hDecoder, &frameInfo, mp4config.bitbuf.data, mp4config.bitbuf.size);
 
         if (adts_out == 1)
         {
@@ -905,32 +910,30 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
             /* write the adts header */
             fwrite(adtsData, 1, adtsDataSize, adtsFile);
 
-            fwrite(buffer, 1, frameInfo.bytesconsumed, adtsFile);
+            fwrite(mp4config.bitbuf.data, 1, frameInfo.bytesconsumed, adtsFile);
         }
 
-        if (buffer) free(buffer);
+        dur = frameInfo.samples / frameInfo.channels;
+        decoded += dur;
+
+        if (decoded > mp4config.samples)
+            dur += mp4config.samples - decoded;
+
+        if (dur > framesize)
+        {
+            faad_fprintf(stderr, "Warning: excess frame detected in MP4 file.\n");
+            dur = framesize;
+        }
 
         if (!noGapless)
         {
-            if (sampleId == 0) dur = 0;
-
-            if (useAacLength || (timescale != samplerate)) {
+            if (useAacLength || (mp4config.samplerate != samplerate)) {
                 sample_count = frameInfo.samples;
             } else {
                 sample_count = (unsigned int)(dur * frameInfo.channels);
                 if (sample_count > frameInfo.samples)
                     sample_count = frameInfo.samples;
-
-                if (!useAacLength && !initial && (sampleId < numSamples/2) && (sample_count != frameInfo.samples))
-                {
-                    faad_fprintf(stderr, "MP4 seems to have incorrect frame duration, using values from AAC data.\n");
-                    useAacLength = 1;
-                    sample_count = frameInfo.samples;
-                }
             }
-
-            if (initial && (sample_count < framesize*frameInfo.channels) && (frameInfo.samples > sample_count))
-                delay = frameInfo.samples - sample_count;
         } else {
             sample_count = frameInfo.samples;
         }
@@ -958,18 +961,14 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
                 if (aufile == NULL)
                 {
                     NeAACDecClose(hDecoder);
-                    mp4ff_close(infile);
-                    free(mp4cb);
-                    fclose(mp4File);
+                    mp4read_close();
                     return 0;
                 }
             }
             first_time = 0;
         }
 
-        if (sample_count > 0) initial = 0;
-
-        percent = min((int)(sampleId*100)/numSamples, 100);
+        percent = min((int)(sampleId*100)/mp4config.frame.ents, 100);
         if (percent > old_percent)
         {
             old_percent = percent;
@@ -1000,14 +999,11 @@ static int decodeMP4file(char *mp4file, char *sndfile, char *adts_fn, int to_std
         fclose(adtsFile);
     }
 
-    mp4ff_close(infile);
+    mp4read_close();
 
     if (!first_time && !adts_out)
         close_audio_file(aufile);
 
-    free(mp4cb);
-    fclose(mp4File);
-
     return frameInfo.error;
 }
 
@@ -1016,6 +1012,7 @@ int main(int argc, char *argv[])
     int result;
     int infoOnly = 0;
     int writeToStdio = 0;
+    int readFromStdin = 0;
     int object_type = LC;
     int def_srate = 0;
     int downMatrix = 0;
@@ -1028,12 +1025,15 @@ int main(int argc, char *argv[])
     int mp4file = 0;
     int noGapless = 0;
     char *fnp;
-    char aacFileName[255];
-    char audioFileName[255];
-    char adtsFileName[255];
+    char *aacFileName = NULL;
+    char *audioFileName = NULL;
+    char *adtsFileName = NULL;
+    float seekTo = 0;
     unsigned char header[8];
     float length = 0;
     FILE *hMP4File;
+    char *faad_id_string;
+    char *faad_copyright_string;
 
 /* System dependant types */
 #ifdef _WIN32
@@ -1063,11 +1063,12 @@ int main(int argc, char *argv[])
             { "info",       0, 0, 'i' },
             { "stdio",      0, 0, 'w' },
             { "stdio",      0, 0, 'g' },
+            { "seek",       1, 0, 'j' },
             { "help",       0, 0, 'h' },
             { 0, 0, 0, 0 }
         };
 
-        c = getopt_long(argc, argv, "o:a:s:f:b:l:wgdhitq",
+        c = getopt_long(argc, argv, "o:a:s:f:b:l:j:wgdhitq",
             long_options, &option_index);
 
         if (c == -1)
@@ -1078,6 +1079,12 @@ int main(int argc, char *argv[])
             if (optarg)
             {
                 outfile_set = 1;
+                audioFileName = (char *) malloc(sizeof(char) * (strlen(optarg) + 1));
+                if (audioFileName == NULL)
+                {
+                    faad_fprintf(stderr, "Error allocating memory for audioFileName.\n");
+                    return 1;
+                }
                 strcpy(audioFileName, optarg);
             }
             break;
@@ -1085,6 +1092,12 @@ int main(int argc, char *argv[])
             if (optarg)
             {
                 adts_out = 1;
+                adtsFileName = (char *) malloc(sizeof(char) * (strlen(optarg) + 1));
+                if (adtsFileName == NULL)
+                {
+                    faad_fprintf(stderr, "Error allocating memory for adtsFileName.\n");
+                    return 1;
+                }
                 strcpy(adtsFileName, optarg);
             }
             break;
@@ -1146,6 +1159,12 @@ int main(int argc, char *argv[])
                 }
             }
             break;
+        case 'j':
+            if (optarg)
+            {
+            	seekTo = atof(optarg);
+            }
+            break;
         case 't':
             old_format = 1;
             break;
@@ -1172,11 +1191,11 @@ int main(int argc, char *argv[])
         }
     }
 
+    NeAACDecGetVersion(&faad_id_string, &faad_copyright_string);
 
-    faad_fprintf(stderr, " *********** Ahead Software MPEG-4 AAC Decoder V%s ******************\n\n", FAAD2_VERSION);
+    faad_fprintf(stderr, " *********** Ahead Software MPEG-4 AAC Decoder V%s ******************\n\n", faad_id_string);
     faad_fprintf(stderr, " Build: %s\n", __DATE__);
-    faad_fprintf(stderr, " Copyright 2002-2004: Ahead Software AG\n");
-    faad_fprintf(stderr, " http://www.audiocoding.com\n");
+    faad_fprintf(stderr, "%s", faad_copyright_string);
     if (cap & FIXED_POINT_CAP)
         faad_fprintf(stderr, " Fixed point version\n");
     else
@@ -1205,6 +1224,12 @@ int main(int argc, char *argv[])
 #endif
 
     /* point to the specified file name */
+    aacFileName = (char *) malloc(sizeof(char) * (strlen(argv[optind]) + 1));
+    if (aacFileName == NULL)
+    {
+        faad_fprintf(stderr, "Error allocating memory for aacFileName.\n");
+        return 1;
+    }
     strcpy(aacFileName, argv[optind]);
 
 #ifdef _WIN32
@@ -1218,6 +1243,12 @@ int main(int argc, char *argv[])
      */
     if(!writeToStdio && !outfile_set)
     {
+        audioFileName = (char *) malloc(sizeof(char) * (strlen(aacFileName) + strlen(file_ext[format]) + 1));
+        if (audioFileName == NULL)
+        {
+            faad_fprintf(stderr, "Error allocating memory for audioFileName.\n");
+            return 1;
+        }
         strcpy(audioFileName, aacFileName);
 
         fnp = (char *)strrchr(audioFileName,'.');
@@ -1229,28 +1260,64 @@ int main(int argc, char *argv[])
     }
 
     /* check for mp4 file */
-    mp4file = 0;
-    hMP4File = fopen(aacFileName, "rb");
-    if (!hMP4File)
-    {
-        faad_fprintf(stderr, "Error opening file: %s\n", aacFileName);
-        return 1;
+    if (0 == strcmp(aacFileName, "-")) {
+   	faad_fprintf(stderr, "Reading from stdin: %s\n", aacFileName);
+	readFromStdin = 1;
+	hMP4File  = stdin;
+#ifdef _WIN32
+        setmode(fileno(stdin), O_BINARY);
+#endif
+
+    } else {
+
+    	mp4file = 0;
+    	hMP4File = fopen(aacFileName, "rb");
+    	if (!hMP4File)
+    	{
+    	    faad_fprintf(stderr, "Error opening file: %s\n", aacFileName);
+    	    return 1;
+    	}
     }
+
     fread(header, 1, 8, hMP4File);
-    fclose(hMP4File);
+
+    if (! readFromStdin )
+      fclose(hMP4File);
+
     if (header[4] == 'f' && header[5] == 't' && header[6] == 'y' && header[7] == 'p')
         mp4file = 1;
 
+    if (!mp4file && seekTo != 0) {
+    	faad_fprintf(stderr, "Warning: can only seek in MP4 files");
+    }
+
     if (mp4file)
     {
         result = decodeMP4file(aacFileName, audioFileName, adtsFileName, writeToStdio,
-            outputFormat, format, downMatrix, noGapless, infoOnly, adts_out, &length);
+            outputFormat, format, downMatrix, noGapless, infoOnly, adts_out, &length, seekTo);
     } else {
+
+	if (readFromStdin == 1) {
+		ungetc(header[7],hMP4File);
+		ungetc(header[6],hMP4File);
+		ungetc(header[5],hMP4File);
+		ungetc(header[4],hMP4File);
+		ungetc(header[3],hMP4File);
+		ungetc(header[2],hMP4File);
+		ungetc(header[1],hMP4File);
+		ungetc(header[0],hMP4File);
+	}
+
         result = decodeAACfile(aacFileName, audioFileName, adtsFileName, writeToStdio,
             def_srate, object_type, outputFormat, format, downMatrix, infoOnly, adts_out,
             old_format, &length);
     }
 
+    if (audioFileName != NULL)
+      free (audioFileName);
+    if (adtsFileName != NULL)
+      free (adtsFileName);
+
     if (!result && !infoOnly)
     {
 #ifdef _WIN32
@@ -1266,5 +1333,8 @@ int main(int argc, char *argv[])
             dec_length, length/dec_length);
     }
 
+    if (aacFileName != NULL)
+      free (aacFileName);
+
     return 0;
 }
